import { z } from "zod";
export const notificationDetailsSchema = z.object({
    url: z.string(),
    token: z.string(),
});
export const eventSchema = z.object({
    header: z.string(),
    payload: z.string(),
    signature: z.string(),
});
// JSON Farcaster Signature header after decoding
export const eventHeaderSchema = z.object({
    fid: z.number(),
    type: z.literal("app_key"),
    key: z.string().startsWith("0x"),
});
// Webhook event payload after decoding
export const eventFrameAddedPayloadSchema = z.object({
    event: z.literal("frame-added"),
    notificationDetails: notificationDetailsSchema.optional(),
});
export const eventFrameRemovedPayloadSchema = z.object({
    event: z.literal("frame-removed"),
});
export const eventNotificationsEnabledPayloadSchema = z.object({
    event: z.literal("notifications-enabled"),
    notificationDetails: notificationDetailsSchema.required(),
});
export const notificationsDisabledPayloadSchema = z.object({
    event: z.literal("notifications-disabled"),
});
export const eventPayloadSchema = z.discriminatedUnion("event", [
    eventFrameAddedPayloadSchema,
    eventFrameRemovedPayloadSchema,
    eventNotificationsEnabledPayloadSchema,
    notificationsDisabledPayloadSchema,
]);
// Notifications API request and response formats
export const sendNotificationRequestSchema = z.object({
    notificationId: z.string().uuid(),
    title: z.string().max(32),
    body: z.string().max(128),
    targetUrl: z.string().max(256),
    tokens: z.string().array().max(100),
});
export const sendNotificationResponseSchema = z.object({
    result: z.object({
        successfulTokens: z.array(z.string()),
        invalidTokens: z.array(z.string()),
        rateLimitedTokens: z.array(z.string()),
    }),
});
